// 後ろで定義するfind関数を使い、
// 入り組んだ配列から「値が文字列であるもの」を探し出す
let array = [
    1, true, [
        2, false, "a"
    ], "b", [
        [
            "c"
        ]
    ]
];

// コールバック関数はアロー関数が便利
const result = find(array, v => typeof v === "string");
console.log(result);
// => [ 'a', 'b', 'c' ]

// モジュールのトップレベルで関数を定義する場合
/**
 * ネストした配列から目的の値を見つけ、集めた結果を返す関数
 * 引数arrayは検索対象の配列、引数condは値が目的のものかチェックする関数
 */
function find(array, cond) {
    const result = []; 
    walkArray(array);
    return result;

    // 巻き上げを利用して補助関数をreturnの後ろに置く
    function walkArray(value) {
        if (Array.isArray(value)) {
            // 配列を見つけたら中身をチェックする
            value.forEach(v => walkArray(v));
        } else if (cond(value)) {
            // 配列以外を見つけたら探し物かどうかチェックし、
            // 目当てのものなら収集する
            result.push(value);
        }
    }
}
