// Object.assignを使うやり方
function repeatA(str, opts) {
    const {times, prefix, postfix} = Object.assign({}, {
        times: 3,
        prefix: "",
        postfix: "",
    }, opts);

    return [...Array(times)].map(() => `${prefix}${str}${postfix}`).join("");
}

console.log(repeatA("Hi"));
// => HiHiHi
console.log(repeatA("Hi", { times: 4 }));
// => HiHiHiHi
console.log(repeatA("Hi", { times: 2, prefix: "!", postfix: "?" }));
// => !Hi?!Hi?


// Object.assignを使わないやり方
function repeatB(str, opts) {
    // 呼び出し元でプロパティの値にnullやundefinedを渡したとき、
    // Object.assignと挙動が違うので注意
    opts = opts || {};
    const times = opts.times || 3;
    const prefix = opts.prefix != null ? opts.prefix : "";
    const postfix = opts.postfix != null ? opts.postfix : "";

    let result = "";
    for (let i = 0; i < times; i++) {
        result += `${prefix}${str}${postfix}`;
    }
    return result;
}

// Object.assignを使ったとき（repeatA）と同じ結果になる
console.log(repeatB("Hi"));
console.log(repeatB("Hi", { times: 4 }));
console.log(repeatB("Hi", { times: 2, prefix: "!", postfix: "?" }));
