// Promiseを使って非同期処理するパターン
function returnAsync1(word) {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve(`promiss ${word}`);
        }, Math.random() * 1000);
    });
}

// コールバック関数を使って非同期処理するパターン
function returnAsync2(word, callback) {
    setTimeout(() => {
        callback(null, `callback ${word}`);
    }, Math.random() * 1000);
}

// それぞれ、3回呼び出して結果を呼び出した順に一度に表示する
{ // Promiseを使う場合
    const returns = [returnAsync1("1"), returnAsync1("2"), returnAsync1("3")];
    Promise.all(returns).then(([r1, r2, r3]) => {
        console.log(`${r1}, ${r2} and ${r3}`);
        // => callback 1, callback 2 and callback 3
    });
}
{ // コールバック関数を使う場合：結果の順序を一定にするのが大変
    const results = [];
    function addResult(idx, v) {
        results[idx] = v;
        if (results.length === 3) {
            const [r1, r2, r3] = results;
            console.log(`${r1}, ${r2} and ${r3}`);
            // => promiss 1, promiss 2 and promiss 3
        }
    }
    returnAsync2("1", (err, v) => addResult(0, v));
    returnAsync2("2", (err, v) => addResult(1, v));
    returnAsync2("3", (err, v) => addResult(2, v));
}
